/*
 * Copyright: Nicholas P Wiebe (2008-2009) and Irmtraud M Meyer (2008-2009)
 * License: licensed under the GNU General Public License version 3 (GPLv3)
 */

#include <algorithm>
#include <cassert>
#include <cstdlib>
#include <iomanip>
#include <iostream>
#include <sstream>
#include <list>
#include <limits>
#include <utility>
#include <ctime>
#include <set>

#include "Alignment.h"
#include "AlignedHelix.h"
#include "HelixFinder.h"
#include "ShuffledAlignment.h"
#include "Utilities.h"

//#include "TF1.h"
//#include "TKDE.h"

extern "C" {
	#include "guugle.c"
}

Realigner HelixFinder::realign = NO_REALIGN;
bool HelixFinder::print_struct = false;
bool HelixFinder::doPvalues = true;
bool HelixFinder::debug_verbose = false;
bool HelixFinder::sample_less = false;
bool HelixFinder::integrate = false;
double HelixFinder::integration_cutoff = 0.01;

//added by Casper:
float HelixFinder::logratiothreshold=-99999;
float HelixFinder::pvaluethreshold=99999;
float HelixFinder::pValueRange = 0.001;


const static int PVALUE_PRECISION = 6; //changed from 2
const static int DEFAULT_PRECISION = 6;
const int CHECK_LIMIT = 10; // Number of consecutive empty random trials allows

HelixFinder::HelixFinder(Alignment * a) : alignment(a){
}

HelixFinder::~HelixFinder() {

}

void HelixFinder::findAllHelices() {

	set<AlignedHelix> helicesSet;

	for (int seqIndex = 0; seqIndex < alignment->seqs.size(); ++seqIndex) {
//	for (int seqIndex = 0; seqIndex < 1; ++seqIndex) {
	
		string name = alignment->seqNames[seqIndex];
		string seq = alignment->seqs[seqIndex];
	//	cout << seq << endl;

		string fasta = ">" + name + "\n" + seq + "\n";
		int file_size = fasta.size();

		char *wholeSequence, *wholeSequence2;
		wholeSequence = (char *) malloc( (file_size + 3) * sizeof(char)); //+3, maybe for 2 #'s and 1 \0.
		// file_size == 99 for "sacCer3.fa"
	//	wholeSequence = new char[file_size + 3];
		strcpy(wholeSequence + 1, fasta.c_str()); // 0 is for '#'
		wholeSequence[file_size + 1]='\0';
		wholeSequence[file_size + 2]='\0'; //TODO stimmt das ?

		wholeSequence2 = (char *) malloc( (file_size + 3) * sizeof(char)); //+3, maybe for 2 #'s and 1 \0.
		memcpy(wholeSequence2, wholeSequence, file_size + 3);

		struct sequenceContainer *seqSetA, *seqSetB;

		seqSetA = parseBuffer(wholeSequence);
		seqSetB = parseBuffer(wholeSequence2);
	//	cout << "test" << endl;
		
		//exit(0);
	//	printf("%s\n",seq1);
	//	printf("A:%s\n",seqSetA->sequence);
	//	printf("B:%s\n\n",seqSetB->sequence);
	//	
//		printf("\n");

		initializeToNum();
		int * suffixesA = getSuffixNumbers(seqSetA);
		int * suffixesB = getSuffixNumbers(seqSetB);
	
		int * resultList;
		struct matchList *matchList = (struct matchList *) malloc(sizeof(struct matchList ));
		matchList->firstMatch=NULL;
		matchList->lastMatch=NULL;

		// main-algorithm:
		int minMatchLength = Alignment::minStemLength;	
		doRecursion(matchList,0, 1, suffixesA[0]-1,1,suffixesB[0]-1,suffixesA,suffixesB,seqSetA->sequence,seqSetB->sequence, minMatchLength);
	
//		printf("\n");
	
		int * count;
		printResults(matchList,seqSetA,seqSetB, count);
	
		free(suffixesA);
		free(suffixesB);

		struct match * match = matchList->firstMatch;

		while (match != NULL) {
			if (match->target < match->query) {
	//			printf("match: %d %d %d\n",match->distance,match->target,match->query);
				int loop = match->query - match->target - 2 * match->distance + 1;
				if (loop < Alignment::minLoopLength) {
					int grow = Alignment::minLoopLength - loop;
					grow += (grow & 1);
					match->distance -= grow / 2;
					if (match->distance < Alignment::minStemLength) {
						match = match->nextMatch;
						continue;
					};
				}
				Helix h(match->target, match->query, match->distance);
				AlignedHelix alignedH(this->alignment, h, seqIndex, *alignment->seq2AlignmentMap[seqIndex]);
				helicesSet.insert(alignedH);
			}
			match = match->nextMatch;
		}
	}

//	cout << "Casting set to vector" << endl;
//	time_t ticks = time(NULL);
	helices.assign(helicesSet.begin(), helicesSet.end());
//	cout << "\tRemoving duplicates " << (double)(time(NULL) - ticks) << " seconds" << endl;
}

void HelixFinder::findAllInteractingHelices(const Alignment & alignmentA, const Alignment & alignmentB) {
	set<AlignedHelix> helicesSet;

	for (int seqIndex = 0; seqIndex < alignment->seqs.size(); ++seqIndex) {
//	for (int seqIndex = 0; seqIndex < 1; ++seqIndex) {
	
		string name = alignment->seqNames[seqIndex];
		string seqA = alignmentA.seqs[seqIndex];
		string seqB = alignmentB.seqs[seqIndex];
//		cout << "A: " << seqA << endl;
//		cout << "B: " << seqB << endl;

		string fastaA = ">" + name + "\n" + seqA + "\n";
		int lengthA = fastaA.size();

		char *wholeSequence, *wholeSequence2;
		wholeSequence = (char *) malloc( (lengthA + 3) * sizeof(char)); //+3, maybe for 2 #'s and 1 \0.
		// lengthA == 99 for "sacCer3.fa"
	//	wholeSequence = new char[lengthA + 3];
		strcpy(wholeSequence + 1, fastaA.c_str()); // 0 is for '#'
		wholeSequence[lengthA + 1]='\0';
		wholeSequence[lengthA + 2]='\0'; //TODO stimmt das ?

		string fastaB = ">" + name + "\n" + seqB + "\n";
		int lengthB = fastaB.size();
		wholeSequence2 = (char *) malloc( (lengthB + 3) * sizeof(char)); //+3, maybe for 2 #'s and 1 \0.
		// lengthB == 99 for "sacCer3.fa"
	//	wholeSequence = new char[lengthB + 3];
		strcpy(wholeSequence2 + 1, fastaB.c_str()); // 0 is for '#'
		wholeSequence2[lengthB + 1]='\0';
		wholeSequence2[lengthB + 2]='\0'; //TODO stimmt das ?

		struct sequenceContainer *seqSetA, *seqSetB;

		seqSetA = parseBuffer(wholeSequence);
		seqSetB = parseBuffer(wholeSequence2);
	//	cout << "test" << endl;
		
		//exit(0);
	//	printf("%s\n",seq1);
	//	printf("A:%s\n",seqSetA->sequence);
	//	printf("B:%s\n\n",seqSetB->sequence);
	//	
//		printf("\n");

		initializeToNum();
		int * suffixesA = getSuffixNumbers(seqSetA);
		int * suffixesB = getSuffixNumbers(seqSetB);
	
		int * resultList;
		struct matchList *matchList = (struct matchList *) malloc(sizeof(struct matchList ));
		matchList->firstMatch=NULL;
		matchList->lastMatch=NULL;

		// main-algorithm:
		int minMatchLength = Alignment::minStemLength;	
		doRecursion(matchList,0, 1, suffixesA[0]-1,1,suffixesB[0]-1,suffixesA,suffixesB,seqSetA->sequence,seqSetB->sequence, minMatchLength);
	
//		printf("\n");
	
		int * count;
		printResults(matchList,seqSetA,seqSetB, count);
	
		free(suffixesA);
		free(suffixesB);

		struct match * match = matchList->firstMatch;

		while (match != NULL) {
			Helix h(match->target, match->query + seqA.size(), match->distance);
			AlignedHelix alignedH(this->alignment, h, seqIndex, *alignment->seq2AlignmentMap[seqIndex]);
			helicesSet.insert(alignedH);
			match = match->nextMatch;
		}
	}

//	cout << "Casting set to vector" << endl;
//	time_t ticks = time(NULL);
	helices.assign(helicesSet.begin(), helicesSet.end());
//	cout << "\tRemoving duplicates " << (double)(time(NULL) - ticks) << " seconds" << endl;
}

void HelixFinder::findAllHelicesOld(){

	//at this point, store helices in bins according to their outer base-pair
	vector<list<AlignedHelix> > helicesByOuterBP;

	//initialize storage container
	for(unsigned int i = 0; i < alignment->length - 1; i++){
		for(unsigned int j = i + 1; j < alignment->length; j++){
			helicesByOuterBP.push_back(list<AlignedHelix>());
			assert(convert(i,j) == (int)helicesByOuterBP.size()- 1);
		}
	}

	//find helices in all sequences
	for(unsigned int i = 0; i < alignment->alignedSeqs.size(); i++){
//	for(unsigned int i = 0; i < 1; i++){
		findAllHelices(i, helicesByOuterBP);
	}

	//store all helices found:
	for(unsigned int j = 0; j < helicesByOuterBP.size(); j++){
		for(list<AlignedHelix>::iterator i = helicesByOuterBP[j].begin(); i != helicesByOuterBP[j].end(); i++){
//			cout << i->dotBracket(alignment->alignedSeqs[0].size()) << "\t" << j << endl;
			helices.push_back(*i);
		}
	}
	//By Casper: call desconstructor of helicesByOuterBP (i.e. delete memory)
	for(unsigned int j =0; j< helicesByOuterBP.size(); j++){
		helicesByOuterBP[j].clear();
	}
	helicesByOuterBP.clear();
}


void HelixFinder::findAllHelices(int seqIndex, vector<list<AlignedHelix> > & helicesByOuterBP){

	int **DP;
	int i, j;

	//double eStack[4][4][4][4];
	//Utilities::ReadStack(eStack);

	string Seq = alignment->seqs[seqIndex];
	int iL = Seq.length();

	// Initialize values

	// allocate and initialize memory for dynamic programming

	DP = (int**) malloc(sizeof(int*) * iL);
	for (i = 0; i < iL; i++) {
		DP[i] = (int*) malloc(sizeof(int) * iL);
	}

	for (i = 0; i < iL; i++) {
		for (j = 0; j < iL; j++) {
			DP[i][j] = 0;
		}
	}

	// dynamic programming

	for (j = MIN_DIST; j < iL; j++) {
		for (i = j - MIN_DIST; i >= 0; i--) {

			assert(i>= 0 && i < iL-MIN_DIST+1);

			if ((Seq[i] == 'A' && Seq[j] == 'U') || (Seq[i] == 'U' && Seq[j] == 'A') || (Seq[i] == 'C' && Seq[j] == 'G') || (Seq[i] == 'G' && Seq[j] == 'C') || (Seq[i] == 'G' && Seq[j] == 'U') || (Seq[i] == 'U' && Seq[j] == 'G')) {
				assert(i+1 < iL-MIN_DIST+1);
				DP[i][j] = DP[i + 1][j - 1] + 1;
			} else {
				DP[i][j] = 0;
			}

			if ((DP[i][j] == 0) && (DP[i + 1][j - 1] >= Alignment::minStemLength)) {

				//found a helix:
				Helix h(i+1, j-1, DP[i+1][j-1]);
//				cout << "Found normal helix: " << h.pos5 << ", " << h.pos3 << "(" << h.length << ")" << endl;
				int loc = convert(alignment->seq2AlignmentMap[seqIndex]->at(i+1), alignment->seq2AlignmentMap[seqIndex]->at(j-1));

				//try to insert it into existing helices;
				bool found = false;
				for(list<AlignedHelix>::iterator it = helicesByOuterBP[loc].begin(); it != helicesByOuterBP[loc].end(); it++){
					found = it->insert(h, seqIndex, *alignment->seq2AlignmentMap[seqIndex]);
					if(found){
						break;
					}
				}

				//not found -> create new AlignedHelix and add it in
				if(!found){
					AlignedHelix alignedH(this->alignment, h, seqIndex, *alignment->seq2AlignmentMap[seqIndex]);
					helicesByOuterBP[loc].push_back(alignedH);
				}

			}
		}
	}

	//find helices on edges of matrix
	for(j = 0; j < iL; j++){

		if (DP[0][j] >= Alignment::minStemLength){
			//found a helix:
			Helix h(0, j, DP[0][j]);
//			cout << "Found edge helix: " << h.pos5 << ", " << h.pos3 << "(" << h.length << ")" << endl;
			int loc = convert(alignment->seq2AlignmentMap[seqIndex]->at(0), alignment->seq2AlignmentMap[seqIndex]->at(j));

			//try to insert it into existing helices;
			bool found = false;
			for(list<AlignedHelix>::iterator it = helicesByOuterBP[loc].begin(); it != helicesByOuterBP[loc].end(); it++){
				found = it->insert(h, seqIndex, *alignment->seq2AlignmentMap[seqIndex]);
				if(found){
					break;
				}
			}

			//not found -> create new AlignedHelix and add it in
			if(!found){
				AlignedHelix alignedH(this->alignment, h, seqIndex, *alignment->seq2AlignmentMap[seqIndex]);
				helicesByOuterBP[loc].push_back(alignedH);
			}

		}

		if (DP[j][iL-1] >= Alignment::minStemLength && j != 0){ //the j==0 case is caught by the above condition (when j = iL-1)
			//found a helix:
			Helix h(j, iL-1, DP[j][iL-1]);
//			cout << "Found edge helix: " << h.pos5 << ", " << h.pos3 << "(" << h.length << ")" << endl;
			int loc = convert(alignment->seq2AlignmentMap[seqIndex]->at(j), alignment->seq2AlignmentMap[seqIndex]->at(iL-1));

			//try to insert it into existing helices;
			bool found = false;
			for(list<AlignedHelix>::iterator it = helicesByOuterBP[loc].begin(); it != helicesByOuterBP[loc].end(); it++){
				found = it->insert(h, seqIndex, *alignment->seq2AlignmentMap[seqIndex]);
				if(found){
					break;
				}
			}

			//not found -> create new AlignedHelix and add it in
			if(!found){
				AlignedHelix alignedH(this->alignment, h, seqIndex, *alignment->seq2AlignmentMap[seqIndex]);
				helicesByOuterBP[loc].push_back(alignedH);
			}
		}

	}

	for (i = 0; i < iL; i++) {
		free(DP[i]);
	}
	free(DP);

	return;
}

int HelixFinder::convert(int pos5, int pos3){
	return pos5 * alignment->length + pos3 - ((pos5+2)*(pos5+1))/2;
}

void HelixFinder::printHelices() {
	ofstream myfile;
	myfile.open("b.txt");
	for(unsigned int j = 0; j < helices.size(); j++){
//		cout << helices[j].dotBracket(alignment->length);
//		cout << "\t" << helices[j].bpsString();
//		cout << "\n";
//		myfile << helices[j].dotBracket(alignment->length) << "\t";
		myfile << helices[j].bpsString();
		myfile << "\n";
	}
}

//This is used only for Transat mode, Interract mode calls a different function.
void HelixFinder::allHelicesPvalueTable(Tree * tree){
	cout << "Pvalue";
	cout << "\t" << "LogLikeRatio";
	cout << "\t" << "PairedLogLikelihood";
	cout << "\t" << "UnpairedLogLikelihood";
	cout << "\t" << "MeanFracCanonicalBP";
	cout << "\t" << "Covariance";
	cout << "\t" << "Conservation";
	cout << "\t" << "HelixLength";
	cout << "\t" << "AlignmentSize";
	cout << "\t" << "AlignmentLength";
	cout << "\t" << "TreeLength";
	cout << "\t" << "Structure";
	cout << "\t" << "Bps";
	cout << endl;

	if(helices.empty()){
		cerr << "No helices found - Pvalue Table empty\n";
		return;
	}

	//initialize pvalues
	vector<double> newPvalues(helices.size(), 0.0);
	vector<double> fastPvalues(helices.size(), 0.0);
	vector<unsigned int> lowerHelixCount(helices.size(), 0);
	unsigned int nullHelixCount = 0;
	vector<double> randLogLikes;
	vector<double> allRandLogLikes;
	//if we don't do p value calculation, p-value column will be all zeros

	Alignment * startingAlignment = this->alignment;

	if(realign == TCOFFEE){
		startingAlignment = alignment->realignTcoffee(tree);
		if (!Alignment::no_cache ) {
			startingAlignment->initializeCache();
		}
	}


	if(doPvalues){

		int minRandomSamples = 5; // Arbitrary cutoff found through empirical test
		unsigned int minRandomHelices = (int) ((1.0 / HelixFinder::pValueRange) * 10);

		if(sample_less) {
			minRandomHelices /= 5;
		}

		if(debug_verbose) cerr << minRandomSamples << " / " << minRandomHelices << endl;
		int i = 0;

		double maxLog = -numeric_limits<double>::infinity();

		int check = 0;
		while (i < minRandomSamples || nullHelixCount < minRandomHelices) {
			if(debug_verbose) cerr << i << " / " << nullHelixCount << endl;
			ShuffledAlignment randomizedAlignment(startingAlignment);
			HelixFinder randomHelices(&randomizedAlignment);

			randomHelices.findAllHelices();
			for(unsigned int j = 0; j < randomHelices.helices.size(); j++){
				double LogLike = randomHelices.helices[j].logLikeRatio(tree);
				randLogLikes.push_back(LogLike);
				if (integrate && LogLike != numeric_limits<double>::infinity()
							&& LogLike != -numeric_limits<double>::infinity()) {
					allRandLogLikes.push_back(LogLike);
				}
			}

			sort(randLogLikes.begin(), randLogLikes.end());

			if(!randLogLikes.empty()){
				int lowerbound;
				for(unsigned int j = 0; j < newPvalues.size(); j++){
					double logLike = helices[j].logLikeRatio(tree);
					lowerbound = Utilities::lowerBound(randLogLikes, 0, randLogLikes.size(), logLike);
					if (logLike > maxLog) maxLog = logLike;
					lowerHelixCount[j] += lowerbound;
				}
			}
			//if empty, implicitly add 0 to each pvalue

			if (randLogLikes.size() == 0){
				++check;
			} else {
				check = 0;
			}
			nullHelixCount += randLogLikes.size();
			randLogLikes.clear();
			++i;

			if(check >= CHECK_LIMIT) {
				cerr << "Too little random helices being generated, aborting" << endl;
				break;
			}
		}

		assert(nullHelixCount >0);

//		TKDE * kde = NULL;
//		TF1 * f1 = NULL;
//		double min_rand_log, max_rand_log = 0;
//		if (integrate) {
//			min_rand_log = *min_element(allRandLogLikes.begin(), allRandLogLikes.end());
//			max_rand_log = *max_element(allRandLogLikes.begin(), allRandLogLikes.end());
//			kde = new TKDE(allRandLogLikes.size(), &allRandLogLikes[0], min_rand_log, max_rand_log, "", 1.0);
//			f1 = new TF1("f1", kde, min_rand_log, maxLog + 1, 0);
//		}
		allRandLogLikes.clear(); //added by Casper

		double cutoff = 1;

		if(debug_verbose) cerr << "Dividing " << newPvalues.size() << " p-values" << endl;

		for(unsigned int j = 0 ; j < newPvalues.size(); j++){
			//take percentage of random helices above real log likelihood
			newPvalues[j] = 1 - lowerHelixCount[j] / (double)nullHelixCount;

			if(debug_verbose) cerr << ".";
//			if(integrate && newPvalues[j] != 0 && newPvalues[j] < cutoff) cutoff = newPvalues[j];
		}
		if(debug_verbose) cerr << endl;

		lowerHelixCount.clear(); //added by Casper

		cutoff *= 100;
		cutoff = min(integration_cutoff, cutoff);

		if(debug_verbose) cerr << "Integrating p-values below: " << cutoff << endl;

//		if (integrate) {
//			for(unsigned int j = 0; j < newPvalues.size(); ++j) {
//				fastPvalues[j] = (newPvalues[j] < cutoff) ? f1->Integral(helices[j].logLikeRatio(tree), maxLog + 1) : newPvalues[j];
//				if(debug_verbose) cerr << ".";
//			}
//			delete kde;
//			delete f1;
//		}

	}

	double treeLength = tree->totalLength();
	int alignmentSize = alignment->seqs.size();

	if(debug_verbose) cerr << "Outputting" << endl;

	for(unsigned int j = 0; j < helices.size(); j++){
		//added by Casper: discard predictions that have a logratio above the specified threshold
		if(logratiothreshold != -99999){
			if(helices[j].logLikeRatio(tree)<logratiothreshold){
					continue;
			}
		}
		if(integrate && !isnan(fastPvalues[j])) {
			if(pvaluethreshold != 99999){
				if(fastPvalues[j]>pvaluethreshold){
						continue;
				}
			}
			cout << scientific << setprecision(PVALUE_PRECISION) << fastPvalues[j] << fixed << setprecision(DEFAULT_PRECISION) << "\t";
		} else {
			if(pvaluethreshold != 99999){
				if(newPvalues[j]>pvaluethreshold){
						continue;
				}
			}
			cout << scientific << setprecision(PVALUE_PRECISION) << newPvalues[j] << fixed << setprecision(DEFAULT_PRECISION) << "\t";
		}
		cout << helices[j].logLikeRatio(tree) << "\t";
		cout << helices[j].pairedLikelihood(tree) << "\t";
		cout << helices[j].unpairedLikelihood(tree) << "\t";
		cout << helices[j].canonicalBP() << "\t";
		cout << helices[j].covariance() << "\t";
		cout << helices[j].conservation() << "\t";
		cout << helices[j].length() << "\t";
		cout << alignmentSize << "\t";
		cout << alignment->length << "\t";
		cout << treeLength << "\t";
		if(print_struct){
			cout << helices[j].dotBracket(alignment->length);
		}
		else{
			cout << 0;
		}
		cout << "\t" << helices[j].bpsString();
		cout << endl;
	}

	if(realign == TCOFFEE){ // only delete alignment here if a new tcoffee alignment was created at the top of the function
		delete startingAlignment;
	}

	return;
}

// TODO: Function currently tasked with splitting A, B and AB helices from interactions.findAllHelices() output...
//This function, as its name suggests, is only called for interaction mode. It doesn't contain the part that actually writes the output
void HelixFinder::allHelicesInteractionsPvalueTable(string & outputFilePrefix, string & helixLogPrefix,
		Tree * tree, Alignment * alignA, Alignment * alignB, HelixFinder &hfAB) {

	if (hfAB.helices.empty()) {
		cout << "No helices found. Pvalue table is empty\n";
		return;
	}

	Alignment* startingAlignmentA = alignA;
	Alignment* startingAlignmentB = alignB;

	std::ofstream* helixLogA = NULL;
	std::ofstream* helixLogB = NULL;
	std::ofstream* helixLogAB = NULL;
	
	bool writeHelixLog;
	if (helixLogPrefix.compare("") == 0) {
		writeHelixLog = false;
	}	
	else {
		writeHelixLog = true;
		string header("Alignment number\tHelix number\tLog Likelihood");
		helixLogA = new std::ofstream((helixLogPrefix + "_A_random_helices.txt").c_str());
		helixLogB = new std::ofstream((helixLogPrefix + "_B_random_helices.txt").c_str());
		helixLogAB = new std::ofstream((helixLogPrefix + "_AB_random_helices.txt").c_str());
		*helixLogA << header << endl;
		*helixLogB << header << endl;
		*helixLogAB << header << endl;
	}

	if(realign == TCOFFEE){
		cout << "\nRealigning data using TCOFFEE\n";

		time_t ticks = time(NULL);
		startingAlignmentA = alignA->realignTcoffee(tree);
		cout << "\t1st input file realigned in " << (double)(time(NULL) - ticks) << " seconds" << endl;

		ticks = time(NULL);
		startingAlignmentB = alignB->realignTcoffee(tree);
		cout << "\t2nd input file realigned in " << (double)(time(NULL) - ticks) << " seconds\n" << endl;
	}
	
	if (!Alignment::no_cache ) {	
		startingAlignmentA->initializeCache();
		startingAlignmentB->initializeCache();
	}

	int boundary = alignA->length;
	unsigned int minRandomHelices = (int) ((1.0 / HelixFinder::pValueRange) * 10);
	unsigned int numTrials = 0;
	unsigned int numTrialsA = 0;
	unsigned int numTrialsB = 0;
	unsigned int numTrialsAB = 0;

	unsigned int minAlignments = 5;
	if(sample_less) minRandomHelices /= 5;

	vector<double> newPvaluesAB(hfAB.helices.size(), 0.0);

	vector<unsigned int> lowerHelixCountAB(hfAB.helices.size(), 0);

	unsigned int nullHelixCountA = 0, nullHelixCountB = 0, nullHelixCountAB = 0;

	vector<double> randLogLikesA, randLogLikesB, randLogLikesAB;
	vector<double> allRandLogLikesA, allRandLogLikesB, allRandLogLikesAB;

	bool continueA = true, continueB = true, continueAB = true;
	double maxLogA = -numeric_limits<double>::infinity();
	double maxLogB = -numeric_limits<double>::infinity();
	double maxLogAB = -numeric_limits<double>::infinity();

	clock_t ticks = clock();

	int checkA = 0, checkB = 0, checkAB = 0;

	if (doPvalues) {

		cout << "p-value range: " << pValueRange << "\nminimum number of random helices required: " << minRandomHelices << "\n\n";

		while(continueA || continueB || continueAB) {
			numTrials++;

			ShuffledAlignment *randomizedAlignmentA = NULL;
			ShuffledAlignment *randomizedAlignmentB = NULL;
			Alignment *concatenatedAlignment = NULL;
	
			if(debug_verbose) cout << "Shuffling" << endl;
	
			if (continueA || continueAB) {
				randomizedAlignmentA = new ShuffledAlignment(startingAlignmentA);
			}
			if (continueB || continueAB) {
				randomizedAlignmentB = new ShuffledAlignment(startingAlignmentB);
			}
			if (continueAB) {
				// Note, this constructor does not shuffle again. It is necessary to preserve the column mapping
				concatenatedAlignment = new ShuffledAlignment(randomizedAlignmentA, randomizedAlignmentB, hfAB.alignment);
			}

			if(debug_verbose) cout << "Finding" << endl;

			if (continueA) {
				HelixFinder randomHelicesA(randomizedAlignmentA);
				randomHelicesA.findAllHelices();

				unsigned int count = HelixFinder::processRandomTrial(randomHelicesA, hfAB, randLogLikesA, allRandLogLikesA,
						lowerHelixCountAB, tree, writeHelixLog, numTrials, nullHelixCountA, maxLogA, helixLogA, boundary, 0);

				nullHelixCountA += count;
				++numTrialsA;

				if (count == 0) {
					++checkA;
				} else {
					checkA = 0;
				}
				if (nullHelixCountA >= minRandomHelices && numTrialsA >= minAlignments)
					continueA = false;
				if (checkA >= CHECK_LIMIT) {
					cerr << numTrialsA << " random alignments generated with little to no helices." << endl;
					cerr << "Too much time wasted attempting to generated Alignment A helices, prematurely aborting helix generation" << endl;
					continueA = false;
				}
			}
			if (continueB) {
				HelixFinder randomHelicesB(randomizedAlignmentB);
				randomHelicesB.findAllHelices();

				unsigned int count = HelixFinder::processRandomTrial(randomHelicesB, hfAB, randLogLikesB, allRandLogLikesB,
						lowerHelixCountAB, tree, writeHelixLog, numTrials, nullHelixCountB, maxLogB, helixLogB, boundary, 1);
				nullHelixCountB += count;
				++numTrialsB;

				if (count == 0) {
					++checkB;
				} else {
					checkB = 0;
				}
				if (nullHelixCountB >= minRandomHelices && numTrialsB >= minAlignments)
					continueB = false;
				if (checkB >= CHECK_LIMIT) {
					cerr << numTrialsB << " random alignments generated with little to no helices." << endl;
					cerr << "Too much time wasted attempting to generated Alignment B helices, prematurely aborting helix generation" << endl;
					continueB = false;
				}
			}
			if (continueAB) {
				HelixFinder randomHelicesAB(concatenatedAlignment);
				randomHelicesAB.findAllHelices();
				randomHelicesAB.filterForABHelices(boundary);

				unsigned int count = HelixFinder::processRandomTrial(randomHelicesAB, hfAB, randLogLikesAB, allRandLogLikesAB,
						lowerHelixCountAB, tree, writeHelixLog, numTrials, nullHelixCountAB, maxLogAB, helixLogAB, boundary, 2);
				nullHelixCountAB += count;
				++numTrialsAB;

				if (count == 0) {
					++checkAB;
				} else {
					checkAB = 0;
				}
				if (nullHelixCountAB >= minRandomHelices && numTrialsAB >= minAlignments)
					continueAB = false;
				if (checkAB >= CHECK_LIMIT) {
					cerr << numTrialsAB << " random alignments generated with little to no helices." << endl;
					cerr << "Too much time wasted attempting to generated Alignment AB helices, prematurely aborting helix generation" << endl;
					continueAB = false;
				}
			}
			if(debug_verbose){
				cout << numTrialsA << ": " << allRandLogLikesA.size() << "\t";
				cout << numTrialsB << ": " << allRandLogLikesB.size() << "\t";
				cout << numTrialsAB << ": " << allRandLogLikesAB.size() << endl;
			}

			randLogLikesA.clear();
			randLogLikesB.clear();
			randLogLikesAB.clear();

			delete randomizedAlignmentA;
			delete randomizedAlignmentB;
			delete concatenatedAlignment;

		}
		cout << "Random trials computed in " << (double)(clock() - ticks) / CLOCKS_PER_SEC << " seconds" << endl;
		cout << nullHelixCountA << " random helices from " << numTrialsA << " trials found in alignment A\n"
				<< nullHelixCountB << " random helices from " << numTrialsB << " trials found in alignment B\n"
				<< nullHelixCountAB << " random helices from " << numTrialsAB << " trials found within interactions\n" << endl;

	}

	if (writeHelixLog) {
		helixLogA->close();
		helixLogB->close();
		helixLogAB->close();
	}

	if (nullHelixCountAB > 0) {

		for (unsigned int i = 0 ; i < newPvaluesAB.size() ; i++) {
				int position = hfAB.helices[i].positionFromBoundary(boundary);
				if (position == LEFT_OF_BOUNDARY) { //if the helix belongs to A alignment
					newPvaluesAB[i] = 1 - lowerHelixCountAB[i] / (double)nullHelixCountA;
				}
				else if (position == RIGHT_OF_BOUNDARY) { //if the helix belongs to B alignment
					newPvaluesAB[i] = 1 - lowerHelixCountAB[i] / (double)nullHelixCountB;
				}
				else if (position == OVERLAPPING_BOUNDARY) { //if the helix belongs to AB alignment
					newPvaluesAB[i] = 1 - lowerHelixCountAB[i] / (double)nullHelixCountAB;
				}
		}

		lowerHelixCountAB.clear(); //added by Casper
	} else {
		cerr << "\tNo random helices, defaulting to 0.0 for all p-values for output" << endl;
	}

	string outFileNameA("_A.out");
	string outFileNameB("_B.out");
	string outFileNameAB("_Interactions.out");
	
	outFileNameA = outputFilePrefix + outFileNameA;
	outFileNameB = outputFilePrefix + outFileNameB;
	outFileNameAB = outputFilePrefix + outFileNameAB;

	cout << "max Log A, B and AB are: "<<maxLogA<<", "<<maxLogB<<", "<<maxLogAB<<endl;
	cout << "rand log likes sizes are: "<<allRandLogLikesA.size()<<", "<<allRandLogLikesB.size()<<", "<<allRandLogLikesAB.size()<<endl;
	cout << "Boundary is: "<<boundary<<endl;

	ticks = clock();
// The HelixFinder::writeHelixData lines were once commented out by Casper for debug purpose
	HelixFinder::writeHelixData(outFileNameA, newPvaluesAB, allRandLogLikesA, hfAB.helices, hfAB.alignment, tree, maxLogA, alignA,-1,boundary,0);
	HelixFinder::writeHelixData(outFileNameB, newPvaluesAB, allRandLogLikesB, hfAB.helices, hfAB.alignment, tree, maxLogB, alignB,-1,boundary,1);
	HelixFinder::writeHelixData(outFileNameAB, newPvaluesAB, allRandLogLikesAB, hfAB.helices, hfAB.alignment, tree, maxLogAB, hfAB.alignment, alignA->length,boundary,2);
	cout << "\nOutput: " << newPvaluesAB.size() << " p-values calculated in " << (double)(clock() - ticks) / CLOCKS_PER_SEC  << " seconds\n" << endl;

	if(realign == TCOFFEE){ // only delete alignment here if a new tcoffee alignment was created at the top of the function
		delete startingAlignmentA;
		delete startingAlignmentB;
	}
}

//this function seems to be only called for interactions. Part of this function is re-used for Transat mode
/* This function is not longer used... should be removed
unsigned int HelixFinder::processRandomTrial(HelixFinder & randomHelices, HelixFinder & realHelices,
		vector<double> & randLogLikes, vector<double> & allRandLogLikes, vector<unsigned int> & lowerHelixCount,
		Tree * tree, bool writeHelixLog, int numTrials, int nullHelixCount, double & maxLog, std::ofstream* log) {

	for(unsigned int j = 0; j < randomHelices.helices.size(); j++){
		double logLikelihood = randomHelices.helices[j].logLikeRatio(tree);
		randLogLikes.push_back(logLikelihood);

		if(logLikelihood != numeric_limits<double>::infinity() && logLikelihood != -numeric_limits<double>::infinity()) {
			allRandLogLikes.push_back(logLikelihood);
		}
		// cout << logLikelihood << endl;
		if (writeHelixLog) {
			int helixNumber = nullHelixCount + j + 1;
			*log << numTrials << "\t" << helixNumber << "\t" << logLikelihood << endl;
		}
	}

	sort(randLogLikes.begin(), randLogLikes.end());

	if(!randLogLikes.empty()){
		int lowerbound;
		for(unsigned int j = 0; j < realHelices.helices.size(); j++){
			double log = realHelices.helices[j].logLikeRatio(tree);
			lowerbound = Utilities::lowerBound(randLogLikes, 0, randLogLikes.size(), log);
			lowerHelixCount[j] += lowerbound;
			if(log > maxLog) maxLog = log;
		}
	}

	unsigned int returnValue = randLogLikes.size();
	randLogLikes.clear();
	return returnValue;
} */

//a copy from above, except  modified by Casper
unsigned int HelixFinder::processRandomTrial(HelixFinder & randomHelices, HelixFinder & realHelices,
		vector<double> & randLogLikes, vector<double> & allRandLogLikes, vector<unsigned int> & lowerHelixCount,
		Tree * tree, bool writeHelixLog, int numTrials, int nullHelixCount, double & maxLog, std::ofstream* log, int boundaryIndex, int a_b_ab) {

	for(unsigned int j = 0; j < randomHelices.helices.size(); j++){
		double logLikelihood = randomHelices.helices[j].logLikeRatio(tree);
		randLogLikes.push_back(logLikelihood);

		if(logLikelihood != numeric_limits<double>::infinity() && logLikelihood != -numeric_limits<double>::infinity()) {
			allRandLogLikes.push_back(logLikelihood);
		}
		// cout << logLikelihood << endl;
		if (writeHelixLog) {
			int helixNumber = nullHelixCount + j + 1;
			*log << numTrials << "\t" << helixNumber << "\t" << logLikelihood << endl;
		}
	}

	sort(randLogLikes.begin(), randLogLikes.end());

	if(!randLogLikes.empty()){
		int lowerbound;
		for(unsigned int j = 0; j < realHelices.helices.size(); j++){
			int position = realHelices.helices[j].positionFromBoundary(boundaryIndex);
			if (position == LEFT_OF_BOUNDARY && a_b_ab==0) { //if the helix belongs to A alignment
				double likelihoodRatio = realHelices.helices[j].logLikeRatio(tree);
				lowerbound = Utilities::lowerBound(randLogLikes, 0, randLogLikes.size(), likelihoodRatio);
				lowerHelixCount[j] += lowerbound;
				if(likelihoodRatio > maxLog) maxLog = likelihoodRatio;
			}
			else if (position == RIGHT_OF_BOUNDARY && a_b_ab==1) { //if the helix belongs to B alignment
				double likelihoodRatio = realHelices.helices[j].logLikeRatio(tree);
				lowerbound = Utilities::lowerBound(randLogLikes, 0, randLogLikes.size(), likelihoodRatio);
				lowerHelixCount[j] += lowerbound;
				if(likelihoodRatio > maxLog) maxLog = likelihoodRatio;
			}
			else if (position == OVERLAPPING_BOUNDARY && a_b_ab==2) { //if the helix belongs to AB alignment
				double likelihoodRatio = realHelices.helices[j].logLikeRatio(tree);
				lowerbound = Utilities::lowerBound(randLogLikes, 0, randLogLikes.size(), likelihoodRatio);
				lowerHelixCount[j] += lowerbound;
				if(likelihoodRatio > maxLog) maxLog = likelihoodRatio;
			}
		}
	}

	unsigned int returnValue = randLogLikes.size();
	randLogLikes.clear();
	return returnValue;
}

void HelixFinder::writeHelixDataMinimal(string filename, Tree * tree) {
	std::ofstream outFile(filename.c_str());
	if (!outFile.is_open()) {
		char *tmpname = strdup("transat_output_XXXXXX");
		mkstemp(tmpname);
		cerr << "Could not create output file (path may not exist/be accessible), writing to file:" << tmpname << endl;
		outFile.close();
		outFile.open(tmpname);
	}
	
	Alignment * true_alignment = this->alignment;
	
	outFile << "Pvalue";
	outFile << "\t" << "LogLikeRatio";
	outFile << "\t" << "PairedLogLikelihood";
	outFile << "\t" << "UnpairedLogLikelihood";
	outFile << "\t" << "MeanFracCanonicalBP";
	outFile << "\t" << "Covariance";
	outFile << "\t" << "Conservation";
	outFile << "\t" << "HelixLength";
	outFile << "\t" << "AlignmentSize";
	outFile << "\t" << "AlignmentLength";
	outFile << "\t" << "TreeLength";
	outFile << "\t" << "Structure";
	outFile << "\t" << "Bps";
//	if (alignALength != -1) outFile << "\t" << "AlignmentLengthA";
	outFile << endl;
	
	double treeLength = tree->totalLength();
	int alignmentSize = true_alignment->seqs.size();
	int alignmentLength = true_alignment->length;

	for (unsigned int i = 0 ; i < helices.size() ; i++) {

		

		// if(debug_verbose) cout << i << " / " << newPvalues.size() << endl;

//		//added by Casper: discard predictions that have a logratio above the specified threshold
//		if(logratiothreshold != -99999){
//			if(helices[i].logLikeRatio(tree)<logratiothreshold){
//					continue;
//			}
//		}
//		if(integrate && newPvalues[i] <= cutoff) {
//			double value = f1->Integral(helices[i].logLikeRatio(tree), maxLog + 1);
//			//added by Casper: discard predictions that have a pvalue above the specified threshold
//			if(pvaluethreshold != 99999){
//				if(value>pvaluethreshold){
//						continue;
//				}
//			}
//			if (isnan(value)) {
//				outFile << scientific << setprecision(PVALUE_PRECISION) << newPvalues[i] << fixed << setprecision(DEFAULT_PRECISION) << "\t";
//				// TODO: Should never happen, but seems to occur randomly... very very rarely --jujubix
//				cerr << "Congratulations, you've triggered a very rare bug, please record input files, settings and the seed and tell the developer" << endl;
//				cerr << "Output should still be usable, although you could rerun the trial without the -fast option if you're worried" << endl;
//			} else {
//				outFile << scientific << setprecision(PVALUE_PRECISION) << value << fixed << setprecision(DEFAULT_PRECISION) << "\t";
//			}
//		} else {
//			//added by Casper: discard predictions that have a pvalue above the specified threshold
//			if(pvaluethreshold != 99999){
//				if(newPvalues[i]>pvaluethreshold){
//						continue;
//				}
//			}
//			outFile << scientific << setprecision(PVALUE_PRECISION) << newPvalues[i] << fixed << setprecision(DEFAULT_PRECISION) << "\t";
//		}
		outFile << scientific << setprecision(PVALUE_PRECISION) << 0.00 << fixed << setprecision(DEFAULT_PRECISION) << "\t";
		outFile << helices[i].logLikeRatio(tree) << "\t";
		outFile << helices[i].pairedLikelihood(tree) << "\t";
		outFile << helices[i].unpairedLikelihood(tree) << "\t";
		outFile << helices[i].canonicalBP() << "\t";
		outFile << helices[i].covariance() << "\t";
		outFile << helices[i].conservation() << "\t";
		outFile << helices[i].length() << "\t";
		outFile << alignmentSize << "\t";
		outFile << alignmentLength << "\t";
		outFile << treeLength << "\t";
		if(print_struct){
			outFile << helices[i].dotBracket(alignmentLength) << "\t";
		} else{
			outFile << 0 << "\t";
		}
		outFile << helices[i].bpsString();
//		if (alignALength != -1) outFile << "\t" << alignALength;
		outFile << endl;
	}
	
	outFile.close();
	
}

void HelixFinder::writeHelixData(string & filename, vector<double> & newPvalues,
		vector<double> & allRandLogLikes, vector<AlignedHelix> & helices, Alignment * alignment, Tree * tree,
		double & maxLog, Alignment * true_alignment, int alignALength, int boundaryIndex, int a_b_ab) {

	if(debug_verbose) cout << "Saving output to: " << filename << endl;

	std::ofstream outFile(filename.c_str());
	if (!outFile.is_open()) {
		char *tmpname = strdup("transat_output_XXXXXX");
		mkstemp(tmpname);
		cerr << "Could not create output file (path may not exist/be accessible), writing to file:" << tmpname << endl;
		outFile.close();
		outFile.open(tmpname);
	}

	outFile << "Pvalue";
	outFile << "\t" << "LogLikeRatio";
	outFile << "\t" << "PairedLogLikelihood";
	outFile << "\t" << "UnpairedLogLikelihood";
	outFile << "\t" << "MeanFracCanonicalBP";
	outFile << "\t" << "Covariance";
	outFile << "\t" << "Conservation";
	outFile << "\t" << "HelixLength";
	outFile << "\t" << "AlignmentSize";
	outFile << "\t" << "AlignmentLength";
	outFile << "\t" << "TreeLength";
	outFile << "\t" << "Structure";
	outFile << "\t" << "Bps";
	if (alignALength != -1) outFile << "\t" << "AlignmentLengthA";
	outFile << endl;

	double min_rand_log = 0, max_rand_log = 0;
	double cutoff = 1;

//	TKDE *kde = NULL;
//	TF1 * f1 = NULL;

//	if (integrate) {
//		for(unsigned int i = 0; i < newPvalues.size(); ++i) {
//				int position = helices[i].positionFromBoundary(boundaryIndex);
//				if (position == LEFT_OF_BOUNDARY && a_b_ab==0) { //if the helix belongs to A alignment
//					if(newPvalues[i] != 0 && newPvalues[i] < cutoff)
//						cutoff = newPvalues[i];
//				}
//				else if (position == RIGHT_OF_BOUNDARY && a_b_ab==1) { //if the helix belongs to B alignment
//					if(newPvalues[i] != 0 && newPvalues[i] < cutoff)
//						cutoff = newPvalues[i];
//				}
//				else if (position == OVERLAPPING_BOUNDARY && a_b_ab==2) { //if the helix belongs to AB alignment
//					if(newPvalues[i] != 0 && newPvalues[i] < cutoff)
//						cutoff = newPvalues[i];
//				}
//		}
//		cutoff *= 100;
//		cutoff = min(integration_cutoff, cutoff);

//		min_rand_log = *min_element(allRandLogLikes.begin(), allRandLogLikes.end());
//		max_rand_log = *max_element(allRandLogLikes.begin(), allRandLogLikes.end());

//		kde = new TKDE(allRandLogLikes.size(), &allRandLogLikes[0], min_rand_log, max_rand_log, "", 1.0);
//		f1 = new TF1("f1", kde, min_rand_log, maxLog + 1, 0);
//	}
	allRandLogLikes.clear();

	double treeLength = tree->totalLength();
	int alignmentSize = true_alignment->seqs.size();
	int alignmentLength = true_alignment->length;

//	cout <<"Size of pval array is: "<<newPvalues.size()<<endl;

	int previous_helix_index;
	for (unsigned int i = 0 ; i < newPvalues.size() ; i++) {
		int position = helices[i].positionFromBoundary(boundaryIndex);

		if(debug_verbose && a_b_ab==0){
			cout << helices[i].logLikeRatio(tree) << "\t";
			cout << helices[i].pairedLikelihood(tree) << "\t";
			cout << helices[i].unpairedLikelihood(tree) << "\t";
			cout << helices[i].canonicalBP() << "\t";
			cout << helices[i].covariance() << "\t";
			cout << "Position: "<<position<<"\t";
			if(a_b_ab==0 && position == RIGHT_OF_BOUNDARY){
				cout << helices[i].bpsString(boundaryIndex);
			}
			else{
				cout << helices[i].bpsString();
			}
			cout <<endl;
		}

		if (position != LEFT_OF_BOUNDARY && a_b_ab==0) { //if looking for alignment A helices, but the current helix doesn't match
			continue;
		}
		if (position != RIGHT_OF_BOUNDARY && a_b_ab==1) { //if looking for alignment B helices, but the current helix doesn't match
			continue;
		}
		if (position != OVERLAPPING_BOUNDARY && a_b_ab==2) { //if looking for alignment AB helices, but the current helix doesn't match
			continue;
		}
		if (position == INVALID_HELIX){
			cerr << "X: " <<  helices[i].bpsString() << endl;
				continue;
		}

		// TODO: This is probably inefficient way to filter out duplicate outputs, if a better way exists, please modify it
//		string previous_helix;
//		if(a_b_ab==1 && i != 0){
//			bool found=false;
//			for (unsigned int j = 0 ; j < i ; j++) {
//				previous_helix=helices[j].bpsString(boundaryIndex);
//				if(helices[i].bpsString(boundaryIndex).compare(previous_helix) ==0){
//					found=true;
//					continue;
//				}
//			}
//			if(found){
//				continue;
//			}
//		}
//		else if(a_b_ab != 1 && i != 0){
//			bool found=false;
//			for (unsigned int j = 0 ; j < i ; j++) {
//				previous_helix=helices[j].bpsString();
//				if(helices[i].bpsString().compare(previous_helix) ==0){
//					found=true;
//					continue;
//				}
//			}
//			if(found){
//				continue;
//			}
//		}

		previous_helix_index=i;

		// if(debug_verbose) cout << i << " / " << newPvalues.size() << endl;

		//added by Casper: discard predictions that have a logratio above the specified threshold
		if(logratiothreshold != -99999){
			if(helices[i].logLikeRatio(tree)<logratiothreshold){
					continue;
			}
		}
//		if(integrate && newPvalues[i] <= cutoff) {
//			double value = f1->Integral(helices[i].logLikeRatio(tree), maxLog + 1);
//			//added by Casper: discard predictions that have a pvalue above the specified threshold
//			if(pvaluethreshold != 99999){
//				if(value>pvaluethreshold){
//						continue;
//				}
//			}
//			if (isnan(value)) {
//				outFile << scientific << setprecision(PVALUE_PRECISION) << newPvalues[i] << fixed << setprecision(DEFAULT_PRECISION) << "\t";
//				// TODO: Should never happen, but seems to occur randomly... very very rarely --jujubix
//				cerr << "Congratulations, you've triggered a very rare bug, please record input files, settings and the seed and tell the developer" << endl;
//				cerr << "Output should still be usable, although you could rerun the trial without the -fast option if you're worried" << endl;
//			} else {
//				outFile << scientific << setprecision(PVALUE_PRECISION) << value << fixed << setprecision(DEFAULT_PRECISION) << "\t";
//			}
//		} else {
			//added by Casper: discard predictions that have a pvalue above the specified threshold
			if(pvaluethreshold != 99999){
				if(newPvalues[i]>pvaluethreshold){
						continue;
				}
			}
			outFile << scientific << setprecision(PVALUE_PRECISION) << newPvalues[i] << fixed << setprecision(DEFAULT_PRECISION) << "\t";
//		}
		outFile << helices[i].logLikeRatio(tree) << "\t";
		outFile << helices[i].pairedLikelihood(tree) << "\t";
		outFile << helices[i].unpairedLikelihood(tree) << "\t";
		outFile << helices[i].canonicalBP() << "\t";
		outFile << helices[i].covariance() << "\t";
		outFile << helices[i].conservation() << "\t";
		outFile << helices[i].length() << "\t";
		outFile << alignmentSize << "\t";
		outFile << alignmentLength << "\t";
		outFile << treeLength << "\t";
		if(print_struct){
			if(a_b_ab == 1){
				outFile << helices[i].dotBracket(alignment->length,boundaryIndex) << "\t";
			}
			else{
				outFile << helices[i].dotBracket(true_alignment->length) << "\t";
			}
		} else{
			outFile << 0 << "\t";
		}
		if(a_b_ab==1){
			outFile << helices[i].bpsString(boundaryIndex);
		}
		else{
			outFile << helices[i].bpsString();
		}
//		if (alignALength != -1) outFile << "\t" << alignALength;
		outFile << endl;
	}

//	if(integrate) {
//		delete kde;
//		delete f1;
//	}

	outFile.close();
}

//old version
void HelixFinder::filterForABHelices(int boundaryIndex) {
	for (unsigned int i = 0 ; i < helices.size(); i++) {
		int position = helices[i].positionFromBoundary(boundaryIndex);
		if (position != OVERLAPPING_BOUNDARY) {
			helices.erase(helices.begin() + i);
			i--;
		}
	}
}

void HelixFinder::findAllHelicesGrow(Tree * tree, double seedThreshold, double growThreshold){

	//matrix that stores which bps score high enough to be considered part of a helix
	//partOfHelix[i][j-i] gives the value for bp i-j (i is 5' of j)
	int alignmentLength = alignment->length;

	vector<vector<bool> > partOfHelix(alignmentLength);
	for(unsigned int i = 0; i < partOfHelix.size(); i++){
		partOfHelix[i] = vector<bool>(partOfHelix.size() - i, false);
	}

	//normalizing factor
	double treeLength = tree->totalLength();

//	list<pair<int, int> > outerGrow;
//	list<pair<int, int> > innerGrow;
	list<pair<int, int> > growPoints;

	//find seed points
	for(int i = 0; i < alignmentLength-MIN_DIST; i++){
		for(int j = i + MIN_DIST; j < alignmentLength; j++){
			if(alignment->logLikelihood(i,j, tree) / treeLength > seedThreshold){
				partOfHelix[i][j-i] = true;
//				if(i > 0 && j+1 < alignment->alignedSeqs[0].size()){
//					outerGrow.push_back(pair<int,int>(i,j));
//				}
//				if(i+MIN_DIST < j){
//					innerGrow.push_back(pair<int,int>(i,j));
//				}
				growPoints.push_back(pair<int,int>(i,j));
			}
		}
	}

	//grow from seed points
	pair<int, int> candidate;
	pair<int, int> current;

	while(!growPoints.empty()){
		current = growPoints.front();
		//outer bp growth:
		if(current.first > 0 && current.second + 1 < alignmentLength){
			candidate = pair<int,int>(current.first -1, current.second +1);
			if(!partOfHelix[candidate.first][candidate.second - candidate.first] && alignment->logLikelihood(candidate.first, candidate.second , tree) / treeLength > growThreshold){
				partOfHelix[candidate.first][candidate.second - candidate.first] = true;
				growPoints.push_back(candidate);
			}
		}


		if(current.first + 1 + MIN_DIST <= current.second - 1){
			candidate = pair<int,int>(current.first +1, current.second -1);
			if(!partOfHelix[candidate.first][candidate.second - candidate.first] && alignment->logLikelihood(candidate.first, candidate.second , tree) / treeLength > growThreshold){
				partOfHelix[candidate.first][candidate.second - candidate.first] = true;
				growPoints.push_back(candidate);
			}
		}
		growPoints.pop_front();
	}

	for(int i = 0; i < alignmentLength; i++){
		for(int k = 0; k < i; k++){
			cout << "\t";
		}
		for(int j = i; j < alignmentLength-1; j++){
			cout << partOfHelix[i][j-i] << "\t";
		}
		cout << partOfHelix[i][alignmentLength-1-i];
		cout << "\n";
	}

	int helixLength = 0;
	for(int i = 0; i < alignmentLength; i++){
		int j;
		for(j = 0; i-j >= 0 && i+j < alignmentLength; j++){
			if(partOfHelix[i-j][2*j]){
				helixLength++;
			}
			else{
				if(helixLength >= Alignment::minStemLength){
					AlignedHelix helix(this->alignment, i-j+1, i+j-1,helixLength);
					helices.push_back(helix);
				}
				helixLength = 0;
			}
		}
		if(helixLength >= Alignment::minStemLength){
			AlignedHelix helix(this->alignment, i-j+1, i+j-1,helixLength);
			helices.push_back(helix);
		}

		helixLength = 0;
	}

}

void HelixFinder::readHelix(string filename) {
	ifstream input(filename.c_str());
	string line;

	if(!input.is_open()){
		cerr << "Error: cannot open helix file " << filename << endl;
		exit(-1);
	}

	getline(input, line); // Comment
	getline(input, line); // Header

	while(input.good()) {
		getline(input, line);
		if (line.empty()) continue;
		stringstream row(line);
		string cell;
		int n = 1;
		int i, j, length;
		while(getline(row, cell, '\t')) {
//			cout << cell << endl;
			if (n == 1) {
				i = atoi(cell.c_str());
			} else if (n == 2) {
				j = atoi(cell.c_str());
			} else if (n == 3) {
				length = atoi(cell.c_str());
			}
			++n;
		}
//		cout << i << ", " << j << " (" << length << ")" << endl;
		helices.push_back(AlignedHelix(alignment, i - 1, j - 1, length));
	}
}

