/*
 * Copyright: Nicholas P Wiebe (2008-2009) and Irmtraud M Meyer (2008-2009)
 * License: licensed under the GNU General Public License version 3 (GPLv3)
 */

#include "Utilities.h"

#include <cassert>
#include <cstdlib>
#include <fstream>
#include <iostream>
#include <sstream>
#include <string>

Utilities::Utilities() {
}

int Utilities::lowerBound(vector<double> & v, int begin, int end, double value){
	int diff = begin-end;
	if (diff == 0){
		return begin;
	}
	else{
		int mid = (begin + end) / 2;
		assert(mid < (int)v.size());
		if(v[mid] < value){
			return Utilities::lowerBound(v, mid+1, end, value);
		}
		else{
			return Utilities::lowerBound(v, begin, mid, value);
		}
	}
}

Utilities::~Utilities() {
}


void Utilities::interpret(char cC, int* table)
{
	table[0] = 0;
	table[1] = 0;
	table[2] = 0;
	table[3] = 0;

	switch(cC) {
		case 'a':
		case 'A':
			table[0] = 1;
			break;
		case 'u':
		case 'U':
		case 't':
		case 'T':
			table[1] = 1;
			break;
		case 'g':
		case 'G':
			table[2] = 1;
			break;
		case 'c':
		case 'C':
			table[3] = 1;
			break;
		case 'b': /* b = g|c|t */
		case 'B':
			table[1] = 1;
			table[2] = 1;
			table[3] = 1;
			break;
		case 'd': /* d = g|a|t */
		case 'D':
			table[0] = 1;
			table[1] = 1;
			table[2] = 1;
			break;
		case 'h': /* h = t|c|a */
		case 'H':
			table[0] = 1;
			table[1] = 1;
			table[3] = 1;
			break;
		case 'v': /* v = g|a|c */
		case 'V':
			table[0] = 1;
			table[2] = 1;
			table[3] = 1;
			break;
		case 'y': /* y = t|c */
		case 'Y':
			table[1] = 1;
			table[3] = 1;
			break;
		case 'r': /* r = a|g */
		case 'R':
			table[0] = 1;
			table[2] = 1;
			break;
		case 'm': /* m = a|c */
		case 'M':
			table[0] = 1;
			table[3] = 1;
			break;
		case 'k': /* k = g|t */
		case 'K':
			table[1] = 1;
			table[2] = 1;
			break;
		case 'w': /* w = a|t */
		case 'W':
			table[0] = 1;
			table[1] = 1;
			break;
		case 's': /* s = g|c */
		case 'S':
			table[2] = 1;
			table[3] = 1;
			break;
		case 'n':
		case 'N':
		case '-':
			table[0] = 1;
			table[1] = 1;
			table[2] = 1;
			table[3] = 1;
			break;
		default:
			cerr << "Error: unrecognized sequence character '" << cC <<"'.\n";
			exit(-1);
	}
}

void Utilities::interpretNonPairingGap(int* nongap, int* table){
	//assert(nongap.size() == 4);
	table[0] = 1;
	table[1] = 1;
	table[2] = 1;
	table[3] = 1;

	if(nongap[0] == 1){
		//A
		table[1] = 0;
	}
	if(nongap[1] == 1){
		//U
		table[0] = 0;
		table[2] = 0;
	}
	if(nongap[2] == 1){
		//G
		table[1] = 0;
		table[3] = 0;
	}
	if(nongap[3] == 1){
		//C
		table[2] = 0;
	}

	//note: special chars 'b', 'd' and 'k' produce an all-zero iTable... So watch out!
}


bool Utilities::validBP(char a, char b){
	//TODO: there are probably quicker ways to implement this
	// maybe convert 'a' and 'b' to indices to access 2D boolean matrix

	b = toupper(b);
	a = toupper(a);

	//note: this is no longer necessary, since conversion is done when
	//sequences are read in.
	if (a == 'T'){
		a = 'U';
	}
	else if(b == 'T'){
		b = 'U';
	}

	string temp("");
	temp += a;
	temp += b;

	//note: compare returns 0 when the strings are equal, hence the !
	if (!temp.compare("AU") || !temp.compare("UA") ){
		return true;
	}

	if (!temp.compare("GU") || !temp.compare("UG")){
		return true;
	}

	if (!temp.compare("GC") || !temp.compare("CG")){
		return true;
	}

	return false;
}

void Utilities::trimSpaces(string& str)
{
	//code from: http://sarathc.wordpress.com/2007/01/31/how-to-trim-leading-or-trailing-spaces-of-string-in-c/
	//changes: now trims endlines as well as spaces

	// Trim Both leading and trailing spaces
	size_t startpos = str.find_first_not_of(" \t\n\r"); // Find the first character position after excluding leading blank spaces
	size_t endpos = str.find_last_not_of(" \t\n\r"); // Find the first character position from reverse af

	// if all spaces or empty return an empty string
	if(( string::npos == startpos ) || ( string::npos == endpos))
	{
		str = "";
	}
	else
		str = str.substr( startpos, endpos-startpos+1 );
	/*
	     // Code for  Trim Leading Spaces only
	     size_t startpos = str.find_first_not_of(� \t�); // Find the first character position after excluding leading blank spaces
	     if( string::npos != startpos )
	         str = str.substr( startpos );
	 */

	/*
	     // Code for Trim trailing Spaces only
	     size_t endpos = str.find_last_not_of(� \t�); // Find the first character position from reverse af
	     if( string::npos != endpos )
	         str = str.substr( 0, endpos+1 );
	 */

}
